/*
 * $Id: auth.c,v 1.4 2003/02/21 12:58:59 tsavola Exp $
 * Timo Savola <timo.savola@movial.fi>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "auth.h"

/**
 * Finds a host from a config file and returns the associated password.
 * If the host has a port defined, it will be stored in the integer pointed
 * to by portp.
 * @param filename the config file path
 * @param host name of the remote machine
 * @param portp pointer to a place where the port number may be stored, or NULL
 * @return the password if found, NULL on error
 */
char *find_pwd(const char *filename,
	       const char *host,
	       int *portp)
{
	FILE *file;
	char *pwd = NULL;

	file = fopen(filename, "r");
	if (!file) {
		return NULL;
	}

	while (!pwd && !feof(file)) {
		char *_host;
		char *_pwd;
		int i;

		i = fscanf(file, "%as %as", &_host, &_pwd);
		if (i >= 2) {
			char *_port;

			/* Split port string from host string. */
			_port = strchr(_host, ':');
			if (_port && strlen(_port) >= 2) {
				*_port++ = '\0';
			}

			/* Is this the correct host? */
			if (strcmp(host, _host) == 0) {
				pwd = _pwd;
				if (_port && portp) {
					*portp = atoi(_port);
				}
			} else {
				free(_pwd);
			}
		}

		if (i >= 1) {
			free(_host);
		}

		if (i < 2) {
			break;
		}
	}

	fclose(file);

	return pwd;
}
