/*
 * $Id: protocol.h,v 1.2 2003/02/21 12:58:59 tsavola Exp $
 * Functions and data structures for client-server communication.
 * Timo Savola <timo.savola@movial.fi>
 */

#ifndef PROTOCOL_H
#define PROTOCOL_H

#include "auth.h"

/** Buffer size for passing stdout/stderr to the client. */
#define BUFSIZE   1024

/** Authentication information. */
typedef struct {
	char user[USER_SIZE];
	char pwd[PWD_SIZE];
} auth_info_t;

/** Information on the command execution. */
typedef struct {
	char *cmd;
	char **argv;
	char *root;
	char *dir;
	char **env;
} cmd_info_t;

extern int write_int16(int fd, int16_t i);
extern int write_str(int fd, const char *str);
extern int write_strv(int fd, const char **strv);

extern int16_t read_int16(int fd);
extern char *read_str(int fd);
extern char **read_strv(int fd);

extern int read_buf(int fd, void *buf, int len);

extern int write_auth(int fd, const char *user, const char *pwd);
extern auth_info_t *read_auth(int fd);

extern int write_cmd(int fd, const char *cmd, const char **argv,
		     const char *root, const char *dir, const char **env);
extern cmd_info_t *read_cmd(int fd);
extern void free_cmd(cmd_info_t *cmd);

extern void free_vec(void **vec);

/** Some data written into stdout by the command. Sent by server. */
#define PTYPE_STDOUT    1

/** Some data written into stderr by the command. Sent by server. */
#define PTYPE_STDERR    2

/** The return code of the command. Sent by server. */
#define PTYPE_RC        3

/** Some data written into stdin for the command. Sent by client. */
#define PTYPE_STDIN     4

/** The stdin (for the command) reached EOF. Sent by client. */
#define PTYPE_STDIN_EOF 5

/** An error message. Sent by server. The client should die after this. */
#define PTYPE_ERROR     6

/** The header of a packet sent by the server. The data follows... */
typedef struct {
	/** One of the PTYPE-values. */
	uint16_t type;
	/** The amount of data. */
	uint32_t size;
} phead_t;

extern int write_packet(int fd, uint16_t type, void *buf, uint32_t size);
extern int read_phead(int fd, phead_t *head);

#endif
